#!#PERL# -w
#
# check_bastille - Nagios plugin which runs Bastille Unix in assessment mode to
#                  generate a report. The score from the assessment is compared
#                  to the previous report. Changes in score will generate a
#                  warning if they go up and a critical alert if they go down.
#                  If the user feels their server is secure, there is an option
#                  to publish the html report to the web for remote viewing.
#
# Copyright (C) 2008  Paul Knight, Delmarva Computer Associates, LLC 
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

use strict;
use warnings;


### Section 1: Configuration
#
# Modify the values of the variables in this section to alter the behavior of
# this plugin.
#


# Bastille Command
#
# Location of the Bastille command.
#
my $cmdBastille = '#BASTILLE#';


# Bastille Arguments
#
# Specify any additional arguments you wish to use include when the assessment
# is run. The default is to only run the assessment, so it is empty.
#
my $argBastille = '';


# Assessment Report Location
#
# Location where Bastille will place it's assessment report.  Change
# this to suit your installation.
#
my $logBastille = '/var/log/Bastille/Assessment/';


# Web Report Location
#
# If you want the assssment report published to your web server, specify the
# directory you want created. This directory, it's contents and any sub-
# directories, will be cleaned and refreshed every time this plugin is run,
# unless --alert is specified. In that case, the directory is only refreshed
# if there has been a change resulting in an alert being raised.
#
# **DO NOT** do this unless the location is secured with SSL encryption and,
# preferrably, some form of authentication. You really do not want everyone to
# know which vulnerabilities you have chosen to leave open. The default prevents
# publishing regardless of the --deploy option.
#
my $webBastille = '';


# Web Report Owner
#
# The user that owns all of the files and directories of the web based
# assessment report.
#
my $webUser = '#ROOT_USER#';


# Web Report Group
#
# The group that owns all of the files and directories of the web based
# assessment report.
#
my $webGroup = '#WEB_GROUP#';



### Section 2: Includes and Subroutines
#
# There are no user configurable parts from here on.
#
use File::Basename;
use File::Find;
use File::Path;
use Nagios::Plugin 0.16 qw (%ERRORS);
use Nagios::Plugin::Functions 0.16 qw(convert);

# Globals
my $nocompare= 0;
my $verbose  = 0;
my $publish  = 0;
my $alert    = 0;
my $warning  = 0;
my $critical = 0;
my $oldScore = 0;
my $oldBase  = 0;
my $newScore = 0;
my $newBase  = 0;
my $rptName  = 'assessment-report';
my %Modules = ();
my %Status = (
    state => "OK",
    reason => "",
);



### Section 3: Initialization
#
# We initialize the main plugin object and related variables. Command line
# options are retrieved and user configuration variables are filtered and
# applied.
#
my $plugin = basename($ENV{NAGIOS_PLUGIN} || $0);
my $np = Nagios::Plugin->new(
    usage => qq(Usage: %s [-v] [-d [-a]] [-w warning-score] [-c critical-score]\n),
    version => '1.0.4',
    url => 'http://www.delmarvacomputer.com/support/nagios/',
    blurb => qq(Runs Bastille in assessment mode, looking for changes.),
    extra => qq(\n
If the latest assessment scores lower than the last one, or if the latest one
scores below the specified critical score, the check will return as critical.
If the latest assessment is higher than the last one, or its score is below the
specified warning score, the check will return a warning. The old and new
reports are also compared line by line for changes, raising a warning if the
scores are the same but individual line items have changed.

Any reduction in hardening is reason for concern. A change could also indicate
suspicious activity. Changes should be confirmed as quickly as possible. If you
recently re-ran Bastille with relaxed answers, or you manually made changes to
system components resulting in a lower score, consider running Bastille again
with more aggressive answers. If you made no changes, you may have sufferred an
intrusion. Consider disconnecting the computer from the network to verify its
integrity.),
);

# Define plugin specific options
$np->add_arg(
    spec => "warning|w=s",
    help => q(-w, --warning=INTEGER
    Exit with WARNING status if score is less than INTEGER out of 10.00
 -w, --warning=PERCENT%
    Exit with WARNING status if score is less than PERCENT));
$np->add_arg(
    spec => "critical|c=s",
    help => q(-c, --critical=INTEGER
    Exit with CRITCAL status if score is less than INTEGER out of 10.00
 -c, --critical=PERCENT%
    Exit with CRITICAL status if score is less than PERCENT));
$np->add_arg(
    spec => "deploy|d",
    help => q(-d, --deploy
    Publish the generated html assessment report to the web server));
$np->add_arg(
    spec => "alert|a",
    help => q(-a, --alert
    Only publish to the web when a change raises either a warning or critcal alert));

# Get command line options
$np->getopts;
my $opts = $np->opts;
$verbose  = $opts->verbose if $opts->verbose;
$publish  = 1 if $opts->deploy;
$alert    = 1 if $opts->alert;
$warning  = $opts->warning if $opts->warning;
$critical = $opts->critical if $opts->critical;

# Strip percent signs from warning/critical
if ($opts->warning && ($warning =~ m/(\D*%\s*)$/)) {
    $warning =~ s/$1$//;
    $warning /= 10.0;
}
if ($opts->critical && ($critical =~ m/(\D*%\s*)$/)) {
    $critical =~ s/$1$//;
    $critical /= 10.0;
}

# Locate the Bastille command and finalize its arguments
$argBastille .= ' --assessnobrowser';
$argBastille =~ s/^\s+//;

# Dump configs and options
print "* Configuring plugin\n" if $verbose;
if ($verbose > 1) {
    print "    User Variables:\n";
    print "        Bastille command is     \'$cmdBastille $argBastille\'\n";
    print "        Bastille Assessment in  \'$logBastille\'\n";
    print "        Deploy web report to    \'$webBastille\'\n";
    print "          files are owned by    ($webUser:$webGroup)\n";
    print "    Options:\n";
    print "        Verbosity               On, level = " . $verbose . "\n";
    print "        Publish to Web          " . ($publish ? "Yes" : "No") . "\n";
    print "          only on alert         " . ($alert ? "Yes" : "No") . "\n";
    print "        Critical threshold      $critical\n";
    print "        Warning threshold       $warning\n";
}

# Validate Bastille command and bail if it's not configured correctly
$np->nagios_exit("UNKNOWN", "cannot find $cmdBastille") unless -f $cmdBastille;# && -x $cmdBastille;


### Section 4: Old Report
#
# Find the old assessment, get the score and details, if necessary, then remove
# it. The report could be in either of the two locations, or it may not exist
# at all. Locate the previous assessment. logBastille takes precedence over
#  webBastille because we may only publish on alert.
#

# Find the directory
my $logDir = '';
if (-e $logBastille) {
    $logDir = $logBastille;
}
elsif (-e $webBastille) {
    $logDir = $webBastille;
}

# Find the report
my $txtReport = "$logDir/$rptName.txt";
$txtReport =~ s!//!/!;
if (-f $txtReport) {
    print "* Processing old report\n" if $verbose;
    print "    Found $txtReport\n" if $verbose > 2;

    # Open the rerport
    if (not open(OLD, "$txtReport")) {
        print "    " if $verbose > 1;
        print "NOTE: Cannot open old report: $!\n";
        last;
    }
    else {
        print "    Successfully openned\n" if $verbose > 2;
    }

    # Load the report
    print "    Parsing report\n" if $verbose > 2;
    my $cntYes = 0;
    my $cntNo  = 0;
    while (<OLD>) {
        chomp;
        next unless ($_ =~ /^[\|S]/);

        if ($_ =~ /^\|/) {
            # Process detail items
            s/^\| //;
            my @fields = split /\|/m;
            my $key    = $fields[0];
            $key =~ s/\s+$//;
            my $value  = $fields[2];
            $value =~ s/^\s+//;
            $value =~ s/\s+$//;
            $Modules{$key} = $value;
            $cntYes++ if ($value =~ /Yes/);
            $cntNo++  if ($value =~ /No/);
        }
        else {
            # Process score
            my @fields = split /:/m;
            ($oldScore, $oldBase) = split(/\//, $fields[1]);
            $oldScore =~ s/^\s+//;
            $oldScore =~ s/\s+$//;
            $oldBase =~ s/^\s+//;
            $oldBase =~ s/\s+$//;
            print "    Score: $oldScore / $oldBase\n" if $verbose > 1;
        }  
    }
    close(OLD);

    print "    Processed " . scalar(keys %Modules) . " items: $cntYes Yes, $cntNo No\n" if $verbose > 1;

}
else {
    $nocompare++;
    print "NOTE: Old report not found\n" if $verbose;
}



### Section 5: New Report
#
# Execute Bastille Unix in assessment only mode. No hardening changes will be
# made. Compare scores and details as requested.
#

# Run Bastille Unix
print "* Preparing assessment\n" if $verbose;
$argBastille .= " 2>&1" if $verbose < 2;
$argBastille .= " 1>/dev/null" if $verbose < 3;
my $status = system("sudo $cmdBastille $argBastille");
if ( $status > 0 ) {
    $np->nagios_exit("UNKNOWN", "Assessment failed ($?): $!");
}
print "* Assessment completed\n" if $verbose;

# Find the directory
if (-e $logBastille) {
    $txtReport = "$logBastille/$rptName.txt";
    $txtReport =~ s!//!/!;
}
else {
    $np->nagios_exit("UNKNOWN", "Assessment directory missing");
}

# Find the report
if (-f $txtReport) {
    print "* Processing new report\n" if $verbose;
    print "    Found $txtReport\n" if $verbose > 2;

    # Open the rerport
    if (not open(NEW, "$txtReport")) {
        print "    " if $verbose > 1;
        print "NOTE: Cannot open new report: $!\n";
        last;
    }
    else {
        print "    Successfully openned\n" if $verbose > 2;
    }

    # Load the report
    print "    Parsing report\n" if $verbose > 2;
    my $cntYes = 0;
    my $cntNo  = 0;
    my $cntNew = 0;
    my $cntRow = 0;
    while (<NEW>) {
        chomp;
        next unless ($_ =~ /^[\|S]/);

        if ($_ =~ /^\|/) {             # Process detail items
            $cntRow++;
            s/^\| //;
            my @fields = split /\|/m;
            my $key    = $fields[0];
            $key =~ s/\s+$//;
            my $value  = $fields[2];
            $value =~ s/^\s+//;
            $value =~ s/\s+$//;
            if ($nocompare) {
                $cntYes++ if ($value =~ /Yes/);
                $cntNo++  if ($value =~ /No/);
            }
            elsif (exists($Modules{$key})) {
                if ($value ne $Modules{$key}) {
                    $cntYes++ if ($value =~ /Yes/);
                    $cntNo++  if ($value =~ /No/);
                    print "        $key: $Modules{$key} -> $value\n" if $verbose > 1;
                }
            }
            else {
                $cntNew++;
                print "        $key: NEW KEY\n" if $verbose > 1;
            }
        }
        else {                         # Process score
            my @fields = split /:/m;
            ($newScore, $newBase) = split(/\//, $fields[1]);
            $newScore =~ s/^\s+//;
            $newScore =~ s/\s+$//;
            $newBase =~ s/^\s+//;
            $newBase =~ s/\s+$//;
            print "    Score: $newScore / $newBase\n" if $verbose > 1;
        }  
    }
    close(NEW);

    if ($nocompare) {
        if ($newScore < $warning) {
            $Status{'state'} = "WARNING";
            $Status{'reason'}= "Score $newScore below $warning";
        }
        elsif ($newScore < $critical) {
            $Status{'state'} = "CRITICAL";
            $Status{'reason'}= "Score $newScore below $critical";
        }
        print "    Processed " . $cntRow . " items: $cntYes Yes, $cntNo No\n" if $verbose > 1;
    }
    else {
        if ($oldScore > $newScore) {
            $Status{'state'} = "CRITICAL";
            $Status{'reason'}= "Score changed from $oldScore to $newScore";
        }
        elsif ($newScore < $critical) {
            $Status{'state'} = "CRITICAL";
            $Status{'reason'}= "Score $newScore below $critical";
        }
        elsif ($newScore < $warning) {
            $Status{'state'} = "WARNING";
            $Status{'reason'}= "Score $newScore below $warning";
        }
        elsif ($oldScore < $newScore) {
            $Status{'state'} = "WARNING";
            $Status{'reason'}= "Score changed from $oldScore to $newScore";
        }
        elsif ($cntYes || $cntNo || $cntNew) {
            $Status{'state'} =   "WARNING";
            $Status{'reason'} =  $cntNo ? "$cntNo changed to No" : "";
            $Status{'reason'} .= $cntYes ? ", $cntYes changed to Yes" : "";
            $Status{'reason'} .= $cntNew ? ", $cntNew added" : "";
        }
        else {
            $Status{'state'} = "OK";
            $Status{'reason'}= "$newScore / $newBase";
        }
        print "    Comparing $cntRow items: $cntYes Yes, $cntNo No, $cntNew New\n" if $verbose > 1;
    }

}
else {
    print "NOTE: New report not found\n" if $verbose;
    $np->nagios_exit("UNKNOWN", "Assessment report missing");
}



### Section 6: Publish
#
# If a location is specified, then publish if indicated. If only publishing on
# an alert, then don't publish unless something changed.
#
my $htmlReport = "$logBastille/$rptName.html";
$htmlReport =~ s!//!/!;

# Find the report
if (-f $htmlReport) {
    print "* Processing HTML Report\n" if $verbose;
    print "    Found $htmlReport\n" if $verbose > 2;

    # Open the rerport
    if (not open(HTML, "+<", $htmlReport)) {
        print "    " if $verbose > 1;
        print "NOTE: Cannot open HTML report: $!\n";
        last;
    }
    else {
        print "    Successfully openned\n" if $verbose > 2;
    }

    # Load the report
    print "    Parsing report\n" if $verbose > 2;
    my $timestamp = localtime();
    my @REPORT = <HTML>;

    # Add the previous score to the report
    for (@REPORT) {
        chomp;
        next unless ($_ =~ /TABLE .*"score"/);
        s!</TD></TR>!</TD><TD><b>Previous Score</b></TD><TD><b>Run Date</b></TD></TR>!;
        s/<\/TD><\/TR>$/<\/TD><TD>$oldScore \/ $oldBase<\/TD><TD>$timestamp<\/TD><\/TR>/;
        last;
    }

    # Write out report
    seek(HTML,0,0);
    for (@REPORT) {
        print HTML "$_\n";
    }
    truncate(HTML,tell(HTML));
    close(HTML);
}


#
# If only publishing on an alert, check the status and change publish
# accordingly
#
if ($publish && $alert) {
    if ($Status{'state'} =~ "OK") {
        $publish = 0;
    }
}


#
# Publish if indicated
#
if ($publish && $webBastille) {

    print "* Publishing report\n" if $verbose;

    # Remove old report
    if ( -d $webBastille ) {
        print "    Removed old report\n" if rmtree( $webBastille ) && $verbose > 2;
    }
    else {
        print "    No old report found\n" if $verbose > 2;
    }

    # Relocate new report
    $status = system( "mv $logBastille $webBastille" );
    die "    Unable to move report: $!\n" if $status;
    print "    Moved report\n" if $verbose > 2;

    # Cleanup ownership and permissions
    my $uid = getpwnam($webUser);
    my $gid = getgrnam($webGroup);
    my $dmode = 0750;
    my $fmode = 0640;
    find sub {
        chown ($uid, $gid, $File::Find::name);
        chmod ($dmode, $File::Find::name) if -d $File::Find::name;
        chmod ($fmode, $File::Find::name) if -f $File::Find::name;;
    }, $webBastille; 
    print "    Permissions changed\n" if $verbose > 2;
}
else {
    print "NOTE: Report will *NOT* be published\n" if $verbose;
}

print "Done.\n" if $verbose > 2;
$np->nagios_exit($Status{'state'}, $Status{'reason'});

1;

# vim:ft=perl:ts=4:sw=4:ai
